# -*- coding: utf-8 -*-

from pynfe.nfe.manual_500 import NFe_310, ProtNFe_310, ProcNFe_310, RetCancNFe_310
from pynfe.nfe.manual_500.processador_nfe import ProcessadorNFe, DANFE
from pynfe.xml_sped.certificado import Certificado
from pynfe.nfe.manual_401 import NFe_200
from os.path import abspath, dirname
from datetime import datetime
from xml.dom import minidom
from OpenSSL import crypto
from io import BytesIO
import pytz


local_tz = pytz.timezone('America/Sao_Paulo')
FILE_DIR = abspath(dirname(__file__))


class NFe(object):
    """
    Especialização da biblioteca pysped.
    """

    @staticmethod
    def extrair_certificado_a1(arquivo, senha):
        """
        Extrai o conteúdo do certificado A1
        """
        pkcs12 = crypto.load_pkcs12(arquivo, senha)
        key = crypto.dump_privatekey(crypto.FILETYPE_PEM, pkcs12.get_privatekey())
        cert = crypto.dump_certificate(crypto.FILETYPE_PEM, pkcs12.get_certificate())
        certificado = Certificado()
        certificado.prepara_certificado_txt(cert)

        return dict(cert=cert, key=key, emissor=certificado.emissor.get('OU'),
                    proprietario=certificado.proprietario.get('CN'),
                    data_inicio_validade=certificado.data_inicio_validade,
                    data_final_validade=certificado.data_fim_validade)

    @staticmethod
    def processar_notas(xml, cert, key, ambiente, estado, contingencia):
        """
        Transmite e processa as notas fiscais eletronicas.
        """
        p = ProcessadorNFe()
        p.ambiente = ambiente
        p.estado = estado
        p.versao = '3.10'
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = True
        p.contingencia = contingencia
        p.caminho = ''
        lista_nfe = list()
        for xml in xml if isinstance(xml, list) else [xml]:
            nfe = NFe_310()
            nfe.infNFe.xml = xml
            lista_nfe.append(nfe)
        return p.processar_notas(lista_nfe)

    @ staticmethod
    def consultar_recibo(lote, cert, key, ambiente, estado, contingencia=False):
        """
        Realiza a consulta do lote de processamento de notas fiscais
        """
        p = ProcessadorNFe()
        p.versao = '3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.contingencia = contingencia
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        return p.consultar_recibo(lote)

    @staticmethod
    def cancelar_nota(chave, protocolo, justificativa, cert, key, ambiente, estado,
                      contingencia=False):
        """
        Realiza o cancelamento da nfe.
        """
        p = ProcessadorNFe()
        p.versao = '3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.contingencia = contingencia
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        return p.cancelar_nota(chave, protocolo, justificativa)

    @staticmethod
    def inutilizar_nota(chave, justificativa, cert, key, ambiente, estado, contingencia=False):
        """
        Realiza a inutilização do número de uma nota fiscal
        """
        p = ProcessadorNFe()
        p.versao = '3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        p.contingencia = contingencia
        p.caminho = ''
        return p.inutilizar_nota(chave, justificativa)

    @staticmethod
    def inutilizar_faixa_numeracao(ano, cnpj, serie, inicio, fim, justificativa, cert, key,
                                   ambiente, estado, contingencia=False):
        """
        Realiza a inutilização de faixa de numeração de nota fiscal
        """
        p = ProcessadorNFe()
        p.versao = '3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        p.contingencia = contingencia
        p.caminho = ''
        return p.inutilizar_nota(ano, cnpj, serie, inicio, fim, justificativa)

    def gerar_danfe(self, xml, retcan_nfe=None, site_emitente='', logo='',
                    nome_sistema='SigERP - www.sigsolucoes.net.br', leiaute_logo_vertical=False):
        """
        Geração do DANFE
        """
        d = DANFE()
        nota = NFe_310()
        prot_nfe = ProtNFe_310()
        proc = ProcNFe_310()
        canc_nfe = RetCancNFe_310()
        nota.xml = xml
        resp = minidom.parseString(xml)
        resp = resp.getElementsByTagName("prot_nfe")[0]
        resposta = resp.toxml()
        prot_nfe.xml = resposta
        proc.protNFe = prot_nfe

        # Danfe de nota fiscal cancelada
        if retcan_nfe:
            resp = minidom.parseString(retcan_nfe)
            resp = resp.getElementsByTagName('retEvento')[0]
            resposta = resp.toxml()
            canc_nfe.xml = resposta

        d.NFe = nota
        d.protNFe = prot_nfe
        d.retCancNFe = canc_nfe
        d.versao = '3.10'
        d.salvar_arquivo = False
        d.obs_impressao = 'DANFE gerado em %s' % self.utc_to_local(datetime.now()).strftime(
            '%d/%m/%Y as %H:%M:%S')
        d.nome_sistema = nome_sistema
        d.site = site_emitente
        d.logo = logo
        d.leiaute_logo_vertical = leiaute_logo_vertical
        d.gerar_danfe()
        danfe_pdf = BytesIO()
        d.danfe.generate_by(PDFGenerator, filename=danfe_pdf)
        d.danfe = danfe_pdf.getvalue()
        danfe_pdf.close()

        return d.danfe

    @staticmethod
    def utc_to_local(utc_dt):
        local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(local_tz)
        return local_tz.normalize(local_dt)

    @staticmethod
    def validar_chave_nfe(chave, uf, data_emissao, cnpj, modelo, serie, numero_nf, considera_cnpj):
        """
        Verifica consistência da chave de NF-e informada.
        """
        msg = ''
        res = {'valida': True, msg: ''}
        if len(chave) == 44:
            if chave[:2] != str(uf):
                msg += '* Estado informado inválido.\n'
            data_emissao = data_emissao[2:4] + data_emissao[5:7]
            if chave[2:6] != data_emissao:
                msg += '* Data de emissão inválida. \n'
            if not considera_cnpj:
                cnpj = chave[6:20]
            cnpj = cnpj.replace('.', '').replace('-', '').replace('/', '')
            if chave[6:20] != cnpj:
                msg += '* CNPJ do emitente inválido. \n'
            if chave[20:22] != str(modelo):
                msg += '* Modelo da Nota Fiscal inválido.\n'
            if chave[22:25] != ('%003d' % int(serie)):
                msg += '* Série da Nota Fiscal inválida.\n'
            if chave[25:34] != '%009d' % int(numero_nf):
                msg += '* Número da Nota Fiscal inválido. \n'
            digito_verificador = NFe_200()._calcula_dv(chave[:43])
            if digito_verificador != int(chave[43:]):
                msg += '* Dígito verificador inválido. \n'
            if msg:
                res['msg'] = 'Chave de acesso inválida: \n' + msg
                res['valida'] = False
        else:
            res['msg'] = 'Chave inválida.'
            res['valida'] = False
        return res

    @staticmethod
    def consultar_nota(chave, cert, key, ambiente, estado, contingencia=False):
        """
        Consulta um nota fiscal
        """
        p = ProcessadorNFe()
        p.versao = '3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.contingencia = contingencia
        p.salvar_arquivos = False
        return p.consultar_nota(chave)

    @staticmethod
    def consultar_cadastro_contribuinte(cpf_cnpj, inscricao_estadual, cert, key, ambiente, estado,
                                        contingencia=False):
        """
        Consulta de cadastro de contribuinte
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.contingencia = contingencia
        p.salvar_arquivos = False
        return p.consultar_cadastro_contribuinte(cpf_cnpj, inscricao_estadual)

    @staticmethod
    def efetuar_manifesto(cnpj, evento, chave, cert, key, ambiente, estado, motivo=None):
        """
        Realiza o manifesto do destinatário
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        return p.manifesto_destinatario(cnpj, evento, chave, motivo)

    @staticmethod
    def consultar_notas_destinatario(cnpj, cert, key, nsu, ambiente, estado):
        """
        Realiza  a consulta do manifesto do destinatário
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        return p.consultar_notas_destinatario(cnpj, nsu)

    @staticmethod
    def download_xml(cnpj, chave, cert, key, ambiente, estado):
        """
        Realiza  a consulta do manifesto do destinatário
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        return p.download_xml(cnpj, chave)

    @staticmethod
    def emitir_carta_correcao(chave, cnpj, correcao, cert, key, sequencia, ambiente, estado):
        """
        Emissao da carta de correcao
        """
        p = ProcessadorNFe()
        p.versao = '3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        return p.corrigir_nota(chave, cnpj, correcao, sequencia)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
